package com.caimei365.tools.task;

import com.caimei365.tools.mapper.LogisticsMapper;
import com.caimei365.tools.model.ResponseJson;
import com.caimei365.tools.model.po.LogisticsInfoPo;
import com.caimei365.tools.service.LogisticsService;
import com.google.gson.Gson;
import com.kuaidi100.sdk.response.QueryTrackResp;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * 物流信息定时查询
 *
 * @author : Charles
 * @date : 2021/5/31
 */
@Slf4j
@Configuration
@EnableScheduling
@RequiredArgsConstructor
public class LogisticsInfoTask {

    private final LogisticsService logisticsService;
    @Resource
    private LogisticsMapper logisticsMapper;
    @Value("${spring.cloud.config.profile}")
    private String profile;

    /**
     * 每小时执行一次查询物流信息
     *
     * cron表达式语法：秒 分 小时 日 月 周
     * 年可省略
     * * 表示所有值。
     * ? 表示不指定值。
     */
    @Scheduled(cron = "0 0 * * * ?")
    public void getLogistics() {
        log.info(">> 定时查询物流信息");
        // 正式环境才去查询物流信息
        if ("prod".equals(profile)) {
            // 近一年未签收物流列表
            List<LogisticsInfoPo> logisticsList = logisticsMapper.getAllUnsignedLogistics();
            if (!CollectionUtils.isEmpty(logisticsList)) {
                logisticsList.forEach(logistics -> {
                    ResponseJson<QueryTrackResp> result = logisticsService.getLogisticsByNumber(logistics.getNu(), logistics.getLogisticsCompanyCode(), "");
                    if (result.getCode() == -2) {
                        // 顺丰速运需要提供收/寄件人的电话号码查询
                        String mobile = logisticsMapper.getLogisticsPhoneByOrderId(logistics.getOrderId());
                        if (StringUtils.isNotEmpty(mobile)) {
                            result = logisticsService.getLogisticsByNumber(logistics.getNu(), logistics.getLogisticsCompanyCode(), mobile);
                        }
                    }
                    if (result.getCode() == 0) {
                        // 查询成功
                        QueryTrackResp queryTrackResp = result.getData();
                        String state = queryTrackResp.getState();
                        String companyCode = queryTrackResp.getCom();
                        String info = new Gson().toJson(queryTrackResp.getData());
                        if (StringUtils.isNotEmpty(state) && StringUtils.isNotEmpty(info)) {
                            logistics.setState(state);
                            logistics.setLogisticsCompanyCode(companyCode);
                            logistics.setInfo(info);
                            logistics.setUpdateDate(new Date());
                            // 更新数据库物流信息
                            logisticsMapper.updateLogistics(logistics);
                            log.info(">> 定时任务 >> 更新物流信息到DB成功！");
                        }
                        if ("400".equals(queryTrackResp.getReturnCode())) {
                            logistics.setState("-1");
                            String errInfo = "[{\"time\":\"" + new Date() + "\",\"context\":\""+queryTrackResp.getMessage()+"\",\"ftime\":\" + new Date() + \"}]";
                            logistics.setInfo(errInfo);
                            logistics.setUpdateDate(new Date());
                            // 更新数据库物流信息
                            logisticsMapper.updateLogistics(logistics);
                            log.info(">> 定时任务 >> 更新物流信息到DB:" + queryTrackResp.getMessage());
                        }
                    }
                });
            }
        }
    }
}
